package net.voltexstudios.entity.custom;

import net.minecraft.entity.EntityType;
import net.minecraft.entity.ai.TargetPredicate;
import net.minecraft.entity.ai.control.FlightMoveControl;
import net.minecraft.entity.ai.goal.*;
import net.minecraft.entity.ai.pathing.BirdNavigation;
import net.minecraft.entity.ai.pathing.EntityNavigation;
import net.minecraft.entity.attribute.DefaultAttributeContainer;
import net.minecraft.entity.attribute.EntityAttributes;
import net.minecraft.entity.mob.HostileEntity;
import net.minecraft.entity.passive.RabbitEntity;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.Heightmap;
import net.minecraft.world.World;

import java.util.EnumSet;
import java.util.Random;

public class VultureEntity extends HostileEntity {

    private static final double STANDARD_FLIGHT_HEIGHT = 20;
    private static final double STANDARD_FLIGHT_SPEED = 1.5;
    private static final double FORWARD_SPEED = 0.2;

    public VultureEntity(EntityType<? extends HostileEntity> entityType, World world) {
        super(entityType, world);
        this.moveControl = new FlightMoveControl(this, 20, true);
    }

    @Override
    protected EntityNavigation createNavigation(World world) {
        BirdNavigation navigation = new BirdNavigation(this, world);
        navigation.setCanPathThroughDoors(false);
        navigation.setCanSwim(true);
        return navigation;
    }

    @Override
    protected void initGoals() {
        this.goalSelector.add(1, new SwimGoal(this));
        this.goalSelector.add(2, new RabbitHunterGoal(this));
        this.goalSelector.add(2, new FlyGoal(this, 2 * STANDARD_FLIGHT_SPEED));

        this.goalSelector.add(3, new LookAroundGoal(this));

        this.goalSelector.add(4, new ForwardFlightGoal(this));
        this.goalSelector.add(4, new MaintainHeightGoal(this, STANDARD_FLIGHT_HEIGHT));


        this.targetSelector.add(2, new ActiveTargetGoal<>(this, RabbitEntity.class, true));
    }

    private static class ForwardFlightGoal extends Goal {
        private final VultureEntity vulture;

        public ForwardFlightGoal(VultureEntity vulture) {
            this.vulture = vulture;
            this.setControls(EnumSet.of(Goal.Control.MOVE));
        }

        @Override
        public boolean canStart() {
            return !vulture.isOnGround();
        }

        @Override
        public void tick() {
            Vec3d rotation = vulture.getRotationVector();
            Vec3d movement = rotation.multiply(FORWARD_SPEED);

            vulture.setVelocity(movement.x, vulture.getVelocity().y, movement.z);
        }
    }

    private static class MaintainHeightGoal extends Goal {
        private final VultureEntity vulture;
        private final double targetHeight;

        public MaintainHeightGoal(VultureEntity vulture, double height) {
            this.vulture = vulture;
            this.targetHeight = height;
        }

        @Override
        public boolean canStart() {
            return true;
        }

        @Override
        public void tick() {
            double currentY = vulture.getY();
            double groundLevel = vulture.getWorld().getTopY(Heightmap.Type.WORLD_SURFACE, vulture.getBlockPos().getX(), vulture.getBlockPos().getZ());
            double desiredY = groundLevel + targetHeight;

            desiredY += new Random().nextDouble() * 5;

            if (Math.abs(currentY - desiredY) > 1.0) {
                vulture.getMoveControl().moveTo(
                        vulture.getX(),
                        desiredY,
                        vulture.getZ(),
                        STANDARD_FLIGHT_SPEED
                );
            }
        }
    }

    @Override
    public void fall(double heightDifference, boolean onGroundIn, net.minecraft.block.BlockState state, net.minecraft.util.math.BlockPos pos) {
        // Kein Fallschaden für den Geier
    }

    public static DefaultAttributeContainer.Builder createVultureAttributes() {
        return HostileEntity.createHostileAttributes()
                .add(EntityAttributes.GENERIC_MAX_HEALTH, 20.0)
                .add(EntityAttributes.GENERIC_MOVEMENT_SPEED, 0.3)
                .add(EntityAttributes.GENERIC_FLYING_SPEED, STANDARD_FLIGHT_SPEED)
                .add(EntityAttributes.GENERIC_ATTACK_DAMAGE, 4.0);
    }

    // Benutzerdefinierte Zielklasse zum Jagen von Hasen
    private static class RabbitHunterGoal extends Goal {
        private final VultureEntity vulture;
        private RabbitEntity targetRabbit;

        public RabbitHunterGoal(VultureEntity vulture) {
            this.vulture = vulture;
        }

        @Override
        public boolean canStart() {
            this.targetRabbit = findNearestRabbit();
            return this.targetRabbit != null;
        }

        @Override
        public void tick() {
            if (targetRabbit != null) {
                // Luftangriff auf den Hasen
                vulture.getNavigation().startMovingTo(
                        targetRabbit.getX(),
                        targetRabbit.getY() + 1.5,
                        targetRabbit.getZ(),
                        1.2
                );

                // Angriff, wenn nah genug
                if (vulture.distanceTo(targetRabbit) < 2.0) {
                    vulture.tryAttack(targetRabbit);
                }
            }
        }

        private RabbitEntity findNearestRabbit() {
            return vulture.getWorld().getClosestEntity(
                    RabbitEntity.class,
                    TargetPredicate.DEFAULT,
                    vulture,
                    vulture.getX(),
                    vulture.getY(),
                    vulture.getZ(),
                    vulture.getBoundingBox().expand(25.0)
            );
        }
    }

    // Benutzerdefinierte Zielklasse für Kreisflugverhalten
    private static class CircularFlightGoal extends Goal {
        private final VultureEntity vulture;
        private double circleX, circleZ;
        private double angle = 0;

        public CircularFlightGoal(VultureEntity vulture) {
            this.vulture = vulture;
            this.setControls(EnumSet.of(Goal.Control.MOVE));
        }

        @Override
        public boolean canStart() {
            // Kreisflug, wenn nicht auf der Jagd
            return !vulture.getNavigation().isIdle();
        }

        @Override
        public void tick() {
            // Kreisfluglogik
            angle += 0.1;
            double radius = 5.0;
            circleX = vulture.getX() + Math.cos(angle) * radius;
            circleZ = vulture.getZ() + Math.sin(angle) * radius;

            vulture.getMoveControl().moveTo(
                    circleX,
                    vulture.getY() + 3,
                    circleZ,
                    STANDARD_FLIGHT_SPEED
            );
        }
    }
}
